library("fda");
##regression spline smoothing
#Y: pxn matrix of original data
#tp: px1 vector of time points within [0,1]
#norder: spline type #default 4 gives cubic splines
#nbreaks: number of interior knots
######nbasis <- norder + nbreaks-2; which should be << min(number of dimension, number of observations)!!!
#fb: use Fourier basis or not
MX2FD_regs <- function(Y,tp,norder=4,nbreaks=20,fb=FALSE)
{
  ##generate basis
  nbasis <- norder + nbreaks-2;#"nbreaks-2" is the number of interior knots
  fb <- TRUE;
  if(fb == TRUE){
    mybasis <- create.fourier.basis(rangeval=c(0,1), nbasis=nbasis, period=diff(c(0,1)));
  }else{
    mybasis <- create.bspline.basis(rangeval=c(0,1), nbasis=nbasis, norder=norder);
  }
  
  ##regression spline smoothing
  #fdata <- Data2fd(argvals=t, y=Y, basisobj=mybasis);#less control, simple regression smoothing
  fdata <- smooth.basis(argvals=tp, y=Y, fdParobj=mybasis)$fd;#same as above, without penalty
  Yfdmx <- eval.fd(evalarg=tp,fdobj=fdata);
  ###Yfdmx_D <- eval.fd(evalarg=tp,fdobj=fdata, Lfdobj=1);#the first derivative
  
  return(Yfdmx);
}

MX2FD_lpk <- function(Y,tp,span=0.75,degree=2)
{
  ncurve <- ncol(Y);
  Yfdmx <- matrix(,length(tp),ncurve);
  for(i in 1:ncurve)
  {
    tmpDF <- data.frame(x=tp,y=Y[,i]);
    fdata <- loess(y ~ x,data=tmpDF,span=span,control=loess.control(surface="direct"),degree=degree);
    Yfdmx[,i] <- predict(fdata,data.frame(x=tp));
  }
  return(Yfdmx);
}

MX2FD_lpk2 <- function(Y,Ytp,Ftp,span=0.75,degree=2)
{
  ncurve <- ncol(Y);
  #nYtp <- nrow(Y);#nYtp <- length(Ytp);
  nFtp <- length(Ftp);
  Yfdmx <- matrix(,nFtp,ncurve);
  for(i in 1:ncurve)
  {
    tmpDF <- data.frame(x=Ytp,y=Y[,i]);
    fdata <- loess(y ~ x,data=tmpDF,span=span,control=loess.control(surface="direct"),degree=degree);
    Yfdmx[,i] <- predict(fdata,data.frame(x=Ftp));
  }
  return(Yfdmx);
}

RUN_EXAMPLE=FALSE;
if(RUN_EXAMPLE)
{
  set.seed(123);
  n <- 10;
  p <- 30#100;
  Y <- matrix(rnorm(n*p),p,n);
  tp <- (0:(p-1))/(p-1);
  
  Ys <- MX2FD_regs(Y,tp,norder=4,nbreaks=6);#check 4+6-2<min(30,10)=10!!!
  
  #plot the smoothed curves vs original discrete data points
  mycolor <- rgb(red=runif(1),green=runif(1),blue=runif(1));
  plot(tp,Ys[,1],type='l',col=mycolor);
  points(tp,Y[,1],col=mycolor);
  for(i in 2:n)
  {
    mycolor <- rgb(red=runif(1),green=runif(1),blue=runif(1));
    lines(tp,Ys[,i],col=mycolor);
    points(tp,Y[,i],col=mycolor);
  }
}